"""
Classes and functions that manage the export of the database entries into .bib files.

This file is part of the physbiblio package.
"""
import os, codecs, re
import bibtexparser
import shutil
import traceback

try:
	from physbiblio.errors import pBLogger
	from physbiblio.config import pbConfig
	from physbiblio.database import pBDB
	from physbiblio.bibtexwriter import pbWriter
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
	print(traceback.format_exc())
	raise

class pbExport():
	"""
	Class that contains the export functions and related.
	"""
	def __init__(self):
		"""
		Initialize the class instance and set some default variables.
		"""
		self.exportForTexFlag = True
		self.backupExtension = ".bck"

	def backupCopy(self, fileName):
		"""
		Creates a backup copy of the given file.

		Parameters:
			fileName: the name of the file to be backed up
		"""
		if os.path.isfile(fileName):
			try:
				shutil.copy2(fileName, fileName + self.backupExtension)
			except IOError:
				pBLogger.exception("Cannot write backup file.\nCheck the folder permissions.")
				return False
			else:
				return True
		return False

	def restoreBackupCopy(self, fileName):
		"""
		Restores the backup copy of the given file, if any.

		Parameters:
			fileName: the name of the file to be restore
		"""
		if os.path.isfile(fileName + self.backupExtension):
			try:
				shutil.copy2(fileName + self.backupExtension, fileName)
			except IOError:
				pBLogger.exception("Cannot restore backup file.\nCheck the file permissions.")
				return False
			else:
				return True
		return False

	def rmBackupCopy(self, fileName):
		"""
		Deletes the backup copy of the given file, if any.

		Parameters:
			fileName: the name of the file of which the backup should be deleted
		"""
		if os.path.isfile(fileName + self.backupExtension):
			try:
				os.remove(fileName + self.backupExtension)
			except IOError:
				pBLogger.exception("Cannot remove backup file.\nCheck the file permissions.")
				return False
			else:
				return True
		return True

	def exportLast(self, fileName):
		"""
		Export the last queried entries into a .bib file, if the list is not empty.

		Parameters:
			fileName: the name of the output bibtex file
		"""
		self.backupCopy(fileName)
		if pBDB.bibs.lastFetched:
			txt = ""
			for q in pBDB.bibs.lastFetched:
				txt += q["bibtex"] + "\n"
			try:
				with codecs.open(fileName, 'w', 'utf-8') as bibfile:
					bibfile.write(txt)
			except Exception:
				pBLogger.exception("Problems in exporting .bib file!")
				self.restoreBackupCopy(fileName)
		else:
			pBLogger.info("No last selection to export!")
		self.rmBackupCopy(fileName)

	def exportAll(self, fileName):
		"""
		Export all the entries in the database into a .bib file.

		Parameters:
			fileName: the name of the output bibtex file
		"""
		self.backupCopy(fileName)
		pBDB.bibs.fetchAll(saveQuery = False, doFetch = False)
		txt = ""
		for q in pBDB.bibs.fetchCursor():
			txt += q["bibtex"] + "\n"
		if txt != "":
			try:
				with codecs.open(fileName, 'w', 'utf-8') as bibfile:
					bibfile.write(txt)
			except Exception:
				pBLogger.exception("Problems in exporting .bib file!", traceback)
				self.restoreBackupCopy(fileName)
		else:
			pBLogger.info("No elements to export!")
		self.rmBackupCopy(fileName)

	def exportSelected(self, fileName, rows):
		"""
		Export the given entries into a .bib file.

		Parameters:
			fileName: the name of the output bibtex file
			rows: the list of entries to be exported
		"""
		self.backupCopy(fileName)
		if len(rows) > 0:
			txt = ""
			for q in rows:
				txt += q["bibtex"] + "\n"
			try:
				with codecs.open(fileName, 'w', 'utf-8') as bibfile:
					bibfile.write(txt)
			except Exception:
				pBLogger.exception("Problems in exporting .bib file!")
				self.restoreBackupCopy(fileName)
		else:
			pBLogger.info("No elements to export!")
		self.rmBackupCopy(fileName)

	def exportForTexFile(self, texFileName, outFileName, overwrite = False, autosave = True):
		"""
		Reads a .tex file looking for the \cite{} commands, collects the bibtex entries cited in the text and stores them in a bibtex file.
		The entries are taken from the database first, or from INSPIRE-HEP if possible.
		The downloaded entries are saved in the database.

		Parameters:
			texFileName: the name (or a list of names) of the considered .tex file(s)
			outFileName: the name of the output file, where the required entries will be added
			overwrite (boolean, default False): if True, the previous version of the file is replaced and no backup copy is created
			autosave (boolean, default True): if True, the changes to the database are automatically saved.

		Output:
			True if successful, False if errors occurred
		"""
		self.exportForTexFlag = True
		pBLogger.info("Reading keys from '%s'"%texFileName)
		pBLogger.info("Saving in '%s'"%outFileName)
		if autosave:
			pBLogger.info("Changes will be automatically saved at the end!")

		if overwrite:
			try:
				with open(outFileName, "w") as o:
					o.write("%file written by PhysBiblio\n")
			except IOError:
				pBLogger.exception("Cannot write on file.\nCheck the file permissions.")

		try:
			existingBib = open(outFileName, "r").read()
		except IOError:
			pBLogger.exception("Cannot read file %s."%outFileName)
			return False

		if type(texFileName) is list:
			if len(texFileName) == 0:
				return False
			for t in texFileName:
				self.exportForTexFile(t, outFileName, overwrite = False, autosave = autosave)
			pBLogger.info("Done for all the texFiles. See previous errors (if any)")
			return True

		cite = re.compile('\\\\(cite|citep|citet)\{([A-Za-z\']*:[0-9]*[a-z]*[,]?[\n ]*|[A-Za-z0-9\-][,]?[\n ]*|[A-Za-z0-9_\-][,]?[\n ]*)*\}', re.MULTILINE)	#find \cite{...}
		unw1 = re.compile('[ ]*(Owner|Timestamp|__markedentry|File)+[ ]*=.*?,[\n]*')	#remove unwanted fields
		unw2 = re.compile('[ ]*(Owner|Timestamp|__markedentry|File)+[ ]*=.*?[\n ]*\}')	#remove unwanted fields
		unw3 = re.compile('[ ]*Abstract[ ]*=[ ]*[{]+(.*?)[}]+,', re.MULTILINE)		#remove Abstract field
		bibel = re.compile('@[a-zA-Z]*\{([A-Za-z]*:[0-9]*[a-z]*)?,', re.MULTILINE | re.DOTALL)	#find the @Article(or other)...}, entry for the key "m"
		bibty = re.compile('@[a-zA-Z]*\{', re.MULTILINE | re.DOTALL)	#find the @Article(or other) entry for the key "m"

		unexpected = []
		def saveEntryOutBib(a):
			"""
			Remove unwanted fields and add the bibtex entry to the output file

			Parameters:
				a: the bibtex entry
			"""
			bib = '@' + a.replace('@', '')
			for u in unw1.finditer(bib):
				bib = bib.replace(u.group(), '')
			for u in unw2.finditer(bib):
				bib = bib.replace(u.group(), '')
			for u in unw3.finditer(bib):
				bib = bib.replace(u.group(), '')
			bibf = '\n'.join([line for line in bib.split('\n') if line.strip() ])
			try:
				with open(outFileName, "a") as o:
					o.write(bibf + "\n")
					pBLogger.info("%s inserted in output file"%m)
			except IOError:
				pBLogger.exception("Impossible to write file '%s'"%outFileName)

		keyscont=""
		try:
			with open(texFileName) as r:
				keyscont += r.read()
		except IOError:
			pBLogger.exception("The file %s does not exist."%texFileName)
			return False

		citaz = [ m for m in cite.finditer(keyscont) if m != "" ]
		pBLogger.info(r"%d \cite commands found in .tex file"%len(citaz))

		requiredBibkeys = []
		for c in citaz:
			b = c.group().replace(r'\cite{', '').replace(r'\citep{', '').replace(r'\citet{', '')
			d = b.replace(' ', '')
			b = d.replace('\n', '')
			d = b.replace(r'}', '')
			a = d.split(',')
			for e in a:
				if e not in requiredBibkeys and e not in existingBib and e.strip() != "":
					requiredBibkeys.append(e)
		pBLogger.info("%d new keys found"%len(requiredBibkeys))

		missing = []
		retrieved = []
		newKeys = {}
		notFound = []
		warnings = 0
		for s in requiredBibkeys:
			if not pBDB.bibs.getByBibtex(s) and s.strip() != "":
				missing.append(s)

		for m in requiredBibkeys:
			if m in missing and self.exportForTexFlag:
				pBLogger.info("Key '%s' missing, trying to import it from Web"%m)
				newWeb = pBDB.bibs.loadAndInsert(m, returnBibtex = True)
				newCheck = pBDB.bibs.getByBibkey(m, saveQuery = False)

				if len(newCheck) > 0:
					pBDB.catBib.insert(pbConfig.params["defaultCategories"], m)
					retrieved.append(m)
					try:
						saveEntryOutBib(pBDB.bibs.getField(m, "bibtex"))
					except:
						unexpected.append(m)
						pBLogger.exception("Unexpected error in saving entry '%s' into the output file"%m)
				else:
					if newWeb and not newWeb.find(m) > 0:
						warnings += 1
						t = [ j.group() for j in bibel.finditer(newWeb) ]
						t1 = []
						for s in t:
							for u in bibty.finditer(s):
								s = s.replace(u.group(), '')
							s = s.replace(',', '')
							t1.append(s)
						newKeys[m] = t1
					else:
						notFound.append(m)
						warnings += 1
				pBLogger.info("\n")
			else:
				try:
					saveEntryOutBib(pBDB.bibs.getField(m, "bibtex"))
				except:
					unexpected.append(m)
					pBLogger.exception("Unexpected error in extracting entry '%s' to the output file"%m)

		if autosave:
			pBDB.commit()
		pBLogger.info("\n\nRESUME")
		pBLogger.info("%d new keys found in .tex file"%len(requiredBibkeys))
		if len(missing) > 0:
			pBLogger.info("%d required entries were missing in database"%len(missing))
		if len(retrieved) > 0:
			pBLogger.info("%d new entries retrieved:"%len(retrieved))
			pBLogger.info(" - ".join(retrieved))
		if len(notFound) > 0:
			pBLogger.info("Impossible to find %d entries:"%len(notFound))
			pBLogger.info(" - ".join(notFound))
		if len(unexpected) > 0:
			pBLogger.info("Unexpected errors for %d entries:"%len(unexpected))
			pBLogger.info(" - ".join(unexpected))
		if len(newKeys.keys()) > 0:
			pBLogger.info("Possible non-matching keys in %d entries"%len(newKeys.keys()) + \
				"\n".join(["'%s' => %s"%(k, ", ".join(n) ) for k, n in newKeys.items() ] ) )
		pBLogger.info("     " + str(warnings) + " warning(s) occurred!")
		return True

	def updateExportedBib(self, fileName, overwrite = False):
		"""
		Reads a bibtex file and updates the entries that it contains, for example if the entry has been published.

		Parameters:
			fileName: the name of the considered bibtex file
			overwrite (boolean, default False): if True, the previous version of the file is replaced and no backup copy is created

		Output:
			True if successful, False if errors occurred
		"""
		self.backupCopy(fileName)
		bibfile=""
		try:
			with open(fileName) as r:
				bibfile += r.read()
		except IOError:
			pBLogger.exception("Cannot write on file.\nCheck the file permissions.")
			return False
		try:
			biblist = bibtexparser.loads(bibfile)
		except IndexError:
			pBLogger.exception("Problems in loading the .bib file!")
			return False
		db = bibtexparser.bibdatabase.BibDatabase()
		db.entries = []
		for b in biblist.entries:
			key = b["ID"]
			element = pBDB.bibs.getByBibkey(key, saveQuery = False)
			if len(element)>0:
				db.entries.append(element[0]["bibtexDict"])
			else:
				db.entries.append(b)
		txt = pbWriter.write(db).strip()
		try:
			with codecs.open(fileName, 'w', 'utf-8') as outfile:
				outfile.write(txt)
		except Exception:
			pBLogger.exception("Problems in exporting .bib file!")
			self.restoreBackupCopy(fileName)
			return False
		if overwrite:
			self.rmBackupCopy(fileName)
		return True

pBExport = pbExport()



#!/usr/bin/env python
"""
Test file for the physbiblio.firstOpen module.

This file is part of the physbiblio package.
"""
import sys, traceback

if sys.version_info[0] < 3:
	import unittest2 as unittest
else:
	import unittest

try:
	from physbiblio.setuptests import *
	from physbiblio.config import pbConfig
	from physbiblio.database import physbiblioDB
	from physbiblio.firstOpen import createTables
except ImportError:
    print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
    raise
except Exception:
	print(traceback.format_exc())

tempFDBName = os.path.join(pbConfig.dataPath, "tests_first_%s.db"%today_ymd)

@unittest.skipIf(skipLongTests, "Long tests")
class TestFirstOpenMethods(unittest.TestCase):
	"""Tests for methods in physbiblio.firstOpen"""

	def test_createTables(self):
		"""Test that all the tables are created at first time, if DB is empty"""
		if os.path.exists(tempFDBName):
			os.remove(tempFDBName)
		open(tempFDBName, 'a').close()
		self.pBDB = physbiblioDB(tempFDBName, noOpen = True)
		self.pBDB.openDB()

		self.assertTrue(self.pBDB.cursExec("SELECT name FROM sqlite_master WHERE type='table';"))
		self.assertEqual([name[0] for name in self.pBDB.cursor()], [])
		createTables(self.pBDB)
		self.assertTrue(self.pBDB.cursExec("SELECT name FROM sqlite_master WHERE type='table';"))
		self.assertEqual(sorted([name[0] for name in self.pBDB.cursor()]), ["categories", "entries", "entryCats", "entryExps", "expCats", "experiments"])
		self.assertTrue([e["name"] for e in self.pBDB.cats.getAll()], ["Main", "Tags"])

		os.remove(tempFDBName)
		open(tempFDBName, 'a').close()
		self.pBDB = physbiblioDB(tempFDBName)
		self.assertTrue(self.pBDB.cursExec("SELECT name FROM sqlite_master WHERE type='table';"))
		self.assertEqual(sorted([name[0] for name in self.pBDB.cursor()]), ["categories", "entries", "entryCats", "entryExps", "expCats", "experiments"])
		self.assertTrue([e["name"] for e in self.pBDB.cats.getAll()], ["Main", "Tags"])

def tearDownModule():
	if os.path.exists(tempFDBName):
		os.remove(tempFDBName)

if __name__=='__main__':
	unittest.main()



"""
Manages the configuration of PhysBiblio, from saving to reading.

This file is part of the physbiblio package.
"""
import sys, ast, os
import logging
import glob
from appdirs import AppDirs

#these are the default parameter values, descriptions and types
configuration_params = [
{"name": "mainDatabaseName",
	"default": 'PBDATAphysbiblio.db',
	"description": 'Name of the database file',
	"special": None},
{"name": "loggingLevel",
	"default": 1,
	"description": 'How many messages to save in the log file (will have effects only after closing the application)',
	"special": 'int'},
{"name": "logFileName",
	"default": 'PBDATAparams.log',
	"description": 'Name of the log file',
	"special": None},
{"name": "pdfFolder",
	"default": 'PBDATApdf',
	"description": 'Folder where to save the PDF files',
	"special": None},
{"name": "pdfApplication",
	"default": '',
	"description": 'Application for opening PDF files',
	"special": None},
{"name": "webApplication",
	"default": '',
	"description": 'Web browser',
	"special": None},
{"name": "timeoutWebSearch",
	"default": 20.,
	"description": 'Timeout for the web queries',
	"special": 'float'},
{"name": "askBeforeExit",
	"default": False,
	"description": 'Confirm before exiting',
	"special": 'boolean'},
{"name": "defaultLimitBibtexs",
	"default": 50,
	"description": 'Number of bibtex entries in the initial view of the main table',
	"special": 'int'},
{"name": "defaultUpdateFrom",
	"default": 0,
	"description": 'Index of bibtex entries (firstdate ASC) from which I should start when using "Update bibtexs"',
	"special": 'int'},
{"name": "maxAuthorNames",
	"default": 3,
	"description": 'Max number of authors to be displayed in the main list',
	"special": 'int'},
{"name": "maxAuthorSave",
	"default": 6,
	"description": 'Max number of authors to be saved when adding info from arXiv',
	"special": 'int'},
{"name": "fetchAbstract",
	"default": False,
	"description": 'Automatically fetch the abstract from arXiv if an arxiv number is present',
	"special": 'boolean'},
{"name": "defaultCategories",
	"default": [],
	"description": 'Default categories for imported bibtexs',
	"special": 'list'},
{"name": "bibListFontSize",
	"default": 9,
	"description": 'Font size in the list of bibtex entries and companion boxes',
	"special": 'float'},
{"name": "bibtexListColumns",
	"default": ["bibkey", "author", "title", "year", "firstdate", "pubdate", "doi", "arxiv", "isbn", "inspire"],
	"description": 'The columns to be shown in the entries list',
	"special": 'list'},
]

loggingLevels = [
	"0 - errors",
	"1 - warnings",
	"2 - info",
	"3 - all"
]

config_paramOrder = [ p["name"] for p in configuration_params ]
config_defaults = {}
config_descriptions = {}
config_special = {}
for p in configuration_params:
	config_defaults[p["name"]] = p["default"]
	config_descriptions[p["name"]] = p["description"]
	config_special[p["name"]] = p["special"]
		
class ConfigVars():
	"""
	Contains all the common settings and the settings stored in the .cfg file.
	Includes also the functions to read and write the config file.
	"""
	def __init__(self):
		"""
		Initialize the configuration.
		Check the profiles first, then for the default profile start with the default parameter values and read the external file.
		"""
		#needed because the main logger will be loaded later!
		logging.basicConfig(format = '[%(module)s.%(funcName)s] %(message)s', level = logging.INFO)
		self.logger = logging.getLogger("physbibliolog")
		self.defaultDirs = AppDirs("PhysBiblio")

		self.configPath = self.defaultDirs.user_config_dir + os.sep
		self.dataPath = self.defaultDirs.user_data_dir + os.sep
		self.logger.info("Default configuration path: %s"%self.configPath)
		if not os.path.exists(self.configPath):
			os.makedirs(self.configPath)
		self.logger.info("Default data path: %s"%self.dataPath)
		if not os.path.exists(self.dataPath):
			os.makedirs(self.dataPath)
		self.configProfilesFile = os.path.join(self.configPath, "profiles.dat")
		self.checkOldProfiles()

		self.needFirstConfiguration = False
		self.paramOrder = config_paramOrder
		self.specialTypes = config_special
		self.defaultsParams = config_defaults
		self.loggingLevels = loggingLevels
		self.params = {}
		for k, v in config_defaults.items():
			if type(v) is str and "PBDATA" in v:
				v = os.path.join(self.dataPath, v.replace("PBDATA", ""))
				config_defaults[k] = v
			self.params[k] = v
		self.descriptions = config_descriptions

		try:
			self.defProf, self.profiles = self.readProfiles()
		except (IOError, ValueError, SyntaxError) as e:
			self.logger.warning(e)
			self.createDefaultProfile()

		self.verifyProfiles()

		self.checkOldPaths()
		for k, prof in self.profiles.items():
			self.profiles[k]["f"] = os.path.join(self.configPath, prof["f"])

		self.defaultProfile = self.profiles[self.defProf]
		self.logger.info("starting with configuration in '%s'"%self.defaultProfile["f"])
		self.configMainFile = self.defaultProfile["f"]
		
		self.readConfigFile()

		#some urls
		self.arxivUrl = "http://arxiv.org/"
		self.doiUrl = "http://dx.doi.org/"
		self.inspireRecord = "http://inspirehep.net/record/"
		self.inspireSearchBase = "http://inspirehep.net/search"

	def verifyProfiles(self):
		"""
		Sync the list of profiles with the config folder content
		"""
		changed = False
		files = [ p["f"].split(os.sep)[-1] for p in self.profiles.values()]
		keys = list(self.profiles.keys())
		for p in keys:
			if not os.path.exists(self.profiles[p]["f"]):
				self.logger.warning("The profile configuration file %s does not exist. Removing profile."%self.profiles[p]["f"])
				del self.profiles[p]
				changed = True
		for f in glob.iglob(self.configPath + os.sep + "*.cfg"):
			s = f.split(os.sep)[-1]
			if s not in files:
				self.logger.warning("New profile configuration file %s found. Adding profile."%f)
				new = s.replace(".cfg", "")
				self.profiles[new] = {"f": f, "d":""}
				changed = True
		if len(self.profiles) == 0:
			self.createDefaultProfile()
			changed = True
		if self.defProf not in self.profiles.keys():
			self.logger.warning("Bad default profile name. Using another existing one.")
			self.defProf = self.profiles.keys()[0]
		if changed:
			self.logger.warning("The list of profiles has been updated to match the content of the config folder.")
			self.writeProfiles()

	def createDefaultProfile(self):
		"""
		Create the default profile
		"""
		self.profiles = {"default": {"f": os.path.join(self.configPath, "params.cfg"), "d":""}}
		open(self.profiles["default"]["f"], "a").close()
		self.defProf = "default"
		self.writeProfiles()

	def readProfiles(self):
		"""
		Reads the list of profiles and the related parameters in the profiles file.
		"""
		with open(self.configProfilesFile) as r:
			txtarr = r.readlines()
		txt = "".join(txtarr)
		return ast.literal_eval(txt.replace("\n",""))

	def checkOldProfiles(self):
		"""
		Intended for backwards compatibility.
		Check if there is a profiles.dat file in the 'data/' subfolder.
		If yes, move it to the new self.configPath.
		"""
		if os.path.isfile(os.path.join("data", "profiles.dat")):
			print("[config] Moving profiles.dat from old to new location")
			os.rename(os.path.join("data", "profiles.dat"), self.configProfilesFile)

	def checkOldPaths(self):
		"""
		Intended for backwards compatibility.
		Check if there are configuration files in the 'data/' subfolder.
		If yes, move them to the new self.configPath or self.dataPath.
		"""
		save = False
		for k, prof in self.profiles.items():
			if "data/" in prof["f"]:
				if os.path.isfile(prof["f"]):
					new = os.path.join(self.configPath, self.profiles[k]["f"].replace("data/", ""))
					print("Moving file %s to %s"%(prof["f"], new))
					os.rename(prof["f"], new)
				self.profiles[k]["f"] = prof["f"].replace("data/", "")
				save = True
		if save:
			self.writeProfiles()

	def writeProfiles(self):
		"""
		Writes the list of profiles and the related parameters in the profiles file.
		"""
		if not os.path.exists(self.configPath):
			os.makedirs(self.configPath)
		with open(self.configProfilesFile, 'w') as w:
			w.write("'%s',\n%s\n"%(self.defProf, self.profiles))

	def reInit(self, newShort, newProfile):
		"""
		Used when changing profile.
		Reloads all the configuration from scratch given the new profile name.

		Parameters:
			newShort (str): short name for the new profile to be loaded
			newProfile (dict): the profile file dictionary
		"""
		self.defProf = newShort
		for k, v in config_defaults.items():
			self.params[k] = v
		self.defaultProfile = newProfile
		self.logger.info("[config] starting with configuration in '%s'"%self.defaultProfile["f"])
		self.configMainFile = self.defaultProfile["f"]
		self.params = {}
		self.readConfigFile()
		
	def readConfigFile(self):
		"""
		Read the configuration from a file, whose name is stored in self.configMainFile.
		Parses the various parameters given their declared type.
		"""
		for k, v in config_defaults.items():
			self.params[k] = v
		try:
			with open(self.configMainFile) as r:
				txt = r.readlines()
			for l in txt:
				k, v = l.replace("\n", "").split(" = ")
				try:
					if config_special[k] == 'float':
						self.params[k] = float(v)
					elif config_special[k] == 'int':
						self.params[k] = int(v)
					elif config_special[k] == 'boolean':
						if v.lower() in ('true','1','yes','on'):
							self.params[k] = True
						elif v.lower() in ('false','0','no','off'):
							self.params[k] = False
						else:
							raise ValueError
					elif config_special[k] == 'list':
						self.params[k] = ast.literal_eval(v.strip())
					else:
						self.params[k] = v
				except Exception:
					self.logger.warning("Failed in reading parameter", exc_info = True)
					self.params[k] = v
		except IOError:
			self.logger.warning("[config] ERROR: config file %s do not exist. Creating it..."%self.configMainFile)
			self.saveConfigFile()
			self.needFirstConfiguration = True
		except Exception:
			self.logger.error("[config] ERROR: reading %s file failed."%self.configMainFile)
		
	def saveConfigFile(self):
		"""
		Write the current configuration in a file, whose name is stored in self.configMainFile.
		"""
		txt = ""
		for k in self.paramOrder:
			current = "%s = %s\n"%(k, self.params[k])
			if current != "%s = %s\n"%(k, config_defaults[k]):
				txt += current
		try:
			with open(self.configMainFile, "w") as w:
				w.write(txt)
		except IOError:
			self.logger.error("[config] ERROR in saving config file!")

pbConfig = ConfigVars()



#!/usr/bin/env python
"""
Test file for the physbiblio.config module.

This file is part of the physbiblio package.
"""
import sys, traceback
import os

if sys.version_info[0] < 3:
	import unittest2 as unittest
	from mock import patch
else:
	import unittest
	from unittest.mock import patch

try:
	from physbiblio.setuptests import *
	from physbiblio.config import pbConfig, ConfigVars, config_defaults
except ImportError:
    print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
    raise
except Exception:
	print(traceback.format_exc())

tempCfgName = os.path.join(pbConfig.dataPath, "tests_%s.cfg"%today_ymd)
tempProfName = os.path.join(pbConfig.dataPath, "tests_%s.dat"%today_ymd)
class TestConfigMethods(unittest.TestCase):
	"""Tests for methods in physbiblio.config"""

	@patch('sys.stdout', new_callable=StringIO)
	def assert_in_stdout_multi(self, function, expected_output, mock_stdout):
		"""Catch and if test stdout of the function contains a string"""
		pBErrorManager.tempHandler(sys.stdout, format = '%(message)s')
		function()
		pBErrorManager.rmTempHandler()
		for e in expected_output:
			self.assertIn(e, mock_stdout.getvalue())

	def test_config(self):
		"""Test config methods for config management"""
		if os.path.exists(tempCfgName):
			os.remove(tempCfgName)
		newConfParamsDict = dict(config_defaults)
		with patch("physbiblio.config.ConfigVars.readProfiles",
				return_value = ("tmp", {"tmp": {"f": tempCfgName, "d":""}})) as _mock_readprof:
			with patch("physbiblio.config.ConfigVars.verifyProfiles", return_value = None) as _verp:
				self.assertFalse(os.path.exists(tempCfgName))
				tempPbConfig = ConfigVars()
				self.assertTrue(os.path.exists(tempCfgName))
				_mock_readprof.assert_called_once_with()
				with open(tempCfgName) as r:
					self.assertEqual(r.readlines(), [])

				self.assertEqual(tempPbConfig.params, newConfParamsDict)
				tempPbConfig.params["logFileName"] = "newfilename"
				newConfParamsDict["logFileName"] = "newfilename"
				tempPbConfig.saveConfigFile()
				with open(tempCfgName) as r:
					self.assertEqual(r.readlines(), ["logFileName = newfilename\n"])

				tempPbConfig1 = ConfigVars()
				self.assertEqual(tempPbConfig1.params, newConfParamsDict)
				tempPbConfig1.params["logFileName"] = "otherfilename"
				newConfParamsDict["logFileName"] = "otherfilename"
				tempPbConfig1.saveConfigFile()
				with open(tempCfgName) as r:
					self.assertEqual(r.readlines(), ["logFileName = otherfilename\n"])

				tempPbConfig.reInit("tmp", {"f": tempCfgName, "d":""})
				self.assertEqual(tempPbConfig.params, newConfParamsDict)
				os.remove(tempCfgName)
				tempPbConfig.reInit("tmp", {"f": tempCfgName, "d":""})
				self.assertEqual(tempPbConfig.params, config_defaults)

				tempPbConfig1.params["timeoutWebSearch"] = 10.
				tempPbConfig1.params["askBeforeExit"] = True
				tempPbConfig1.params["maxAuthorNames"] = 5
				tempPbConfig1.params["defaultCategories"] = [1]
				tempPbConfig1.saveConfigFile()
				with open(tempCfgName) as r:
					self.assertEqual(r.readlines(),
						["logFileName = otherfilename\n",
						"timeoutWebSearch = 10.0\n",
						"askBeforeExit = True\n",
						"maxAuthorNames = 5\n",
						"defaultCategories = [1]\n"])
				tempPbConfig.reInit("tmp", {"f": tempCfgName, "d":""})
				self.assertEqual(tempPbConfig.params, tempPbConfig1.params)

				os.remove(tempCfgName)
				self.assertFalse(tempPbConfig1.needFirstConfiguration)
				tempPbConfig1.readConfigFile()
				self.assertTrue(tempPbConfig1.needFirstConfiguration)

	def test_profiles(self):
		"""Test config methods for profiles management"""
		tempPbConfig = ConfigVars()
		if os.path.exists(tempProfName):
			os.remove(tempProfName)
		origDef, origProfiles = tempPbConfig.defProf, tempPbConfig.profiles

		tempPbConfig.defProf, tempPbConfig.profiles = ("tmp", {"tmp": {"f": tempCfgName, "d":""}})
		tempPbConfig.configProfilesFile = tempProfName
		tempPbConfig.writeProfiles()
		with open(tempProfName) as r:
			lines = r.readlines()
		self.assertEqual(lines[0], "'tmp',\n")
		self.assertIn(lines[1],
			["{'tmp': {'f': '%s', 'd': ''}}\n"%tempCfgName,
			"{'tmp': {'d': '', 'f': '%s'}}\n"%tempCfgName])

		tempPbConfig.defProf, tempPbConfig.profiles = origDef, origProfiles
		self.assertEqual(tempPbConfig.defProf, origDef)
		self.assertEqual(tempPbConfig.profiles, origProfiles)
		tempPbConfig.defProf, tempPbConfig.profiles = tempPbConfig.readProfiles()
		self.assertEqual(tempPbConfig.defProf, "tmp")
		self.assertEqual(tempPbConfig.profiles, {"tmp": {"f": tempCfgName, "d":""}})

		tempPbConfig.defProf, tempPbConfig.profiles = ("new", {"new": {"f": tempCfgName + "abc", "d":""}})
		with patch("physbiblio.config.ConfigVars.writeProfiles", return_value = None) as _verp:
			self.assert_in_stdout_multi(tempPbConfig.verifyProfiles,
				["Bad default profile name. Using another existing one.",
				"The profile configuration file %s does not exist. Removing profile."%(tempCfgName + "abc"),
				"The list of profiles has been updated to match the content of the config folder."])

def tearDownModule():
	if os.path.exists(tempCfgName):
		os.remove(tempCfgName)
	if os.path.exists(tempProfName):
		os.remove(tempProfName)

if __name__=='__main__':
	unittest.main()



#!/usr/bin/env python
"""
Test file for the physbiblio.export module.

This file is part of the physbiblio package.
"""
import sys, traceback, os
from stat import S_IREAD, S_IRGRP, S_IROTH

if sys.version_info[0] < 3:
	import unittest2 as unittest
	from mock import patch
else:
	import unittest
	from unittest.mock import patch

try:
	from physbiblio.setuptests import *
	from physbiblio.config import pbConfig
	from physbiblio.database import pBDB
	from physbiblio.export import pBExport
except ImportError:
    print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
    raise
except Exception:
	print(traceback.format_exc())

@unittest.skipIf(skipLongTests, "Long tests")
class TestExportMethods(unittest.TestCase):
	"""Tests for methods in physbiblio.export"""
	def test_backup(self):
		"""Test backup file creation and related"""
		emptyFileName = os.path.join(pbConfig.dataPath, "tests_%s.bib"%today_ymd)
		if os.path.exists(emptyFileName): os.remove(emptyFileName)
		if os.path.exists(emptyFileName + pBExport.backupExtension): os.remove(emptyFileName + pBExport.backupExtension)
		self.assertFalse(pBExport.backupCopy(emptyFileName))
		self.assertFalse(os.path.exists(emptyFileName))
		self.assertFalse(os.path.exists(emptyFileName + pBExport.backupExtension))
		open(emptyFileName, 'a').close()
		self.assertTrue(pBExport.backupCopy(emptyFileName))
		self.assertTrue(os.path.exists(emptyFileName + pBExport.backupExtension))
		os.remove(emptyFileName)
		self.assertFalse(os.path.exists(emptyFileName))
		self.assertTrue(pBExport.restoreBackupCopy(emptyFileName))
		self.assertTrue(os.path.exists(emptyFileName))
		os.chmod(emptyFileName, S_IREAD)
		self.assertFalse(pBExport.restoreBackupCopy(emptyFileName))
		os.remove(emptyFileName)
		self.assertTrue(pBExport.rmBackupCopy(emptyFileName))
		self.assertFalse(os.path.exists(emptyFileName + pBExport.backupExtension))

	def test_offlineExports(self):
		"""Test of offline export functions exportSelected, updateExportedBib"""
		testBibName = os.path.join(pbConfig.dataPath, "tests_%s.bib"%today_ymd)
		sampleList = [{"bibtex": '@Article{empty,\nauthor="me",\ntitle="no"\n}'}, {"bibtex": '@Article{empty2,\nauthor="me2",\ntitle="yes"\n}'}]
		sampleTxt = '@Article{empty,\nauthor="me",\ntitle="no"\n}\n@Article{empty2,\nauthor="me2",\ntitle="yes"\n}\n'
		pBDB.bibs.lastFetched = sampleList
		pBExport.exportLast(testBibName)
		self.assertEqual(open(testBibName).read(), sampleTxt)
		os.remove(testBibName)

		sampleList = [{"bibtex": '@Article{empty2,\nauthor="me2",\ntitle="{yes}",\n}'}]
		sampleTxt = '@Article{empty2,\nauthor="me2",\ntitle="{yes}",\n}\n'
		pBExport.exportSelected(testBibName, sampleList)
		self.assertEqual(open(testBibName).read(), sampleTxt)

		with patch('physbiblio.database.entries.getByBibkey', return_value = [{"bibtexDict": {"ID": "empty2", "ENTRYTYPE": "Article", "author": "me et al", "title": "yes"}}]) as _mock:
			pBExport.updateExportedBib(testBibName, overwrite = True)
		self.assertEqual(open(testBibName).read().replace(" ","").replace("\n",""),
			 sampleTxt.replace("me2", 'me et al').replace(" ","").replace("\n",""))
		pBExport.rmBackupCopy(testBibName)
		os.remove(testBibName)

	def test_exportAll(self):
		"""Test of exportAll"""
		testBibName = os.path.join(pbConfig.dataPath, "tests_%s.bib"%today_ymd)
		sampleList = [{"bibtex": '@Article{empty,\nauthor="me",\ntitle="no"\n}'}, {"bibtex": '@Article{empty2,\nauthor="me2",\ntitle="yes"\n}'}]
		sampleTxt = '@Article{empty,\nauthor="me",\ntitle="no"\n}\n@Article{empty2,\nauthor="me2",\ntitle="yes"\n}\n'
		with patch('physbiblio.database.entries.fetchCursor', return_value = sampleList) as _curs:
			pBExport.exportAll(testBibName)
		self.assertEqual(open(testBibName).read(), sampleTxt)
		os.remove(testBibName)

	def test_exportForTexFile(self):
		"""test exportForTexFile function with a fake tex and database"""
		testBibName = os.path.join(pbConfig.dataPath, "tests_%s.bib"%today_ymd)
		self.assertFalse(os.path.exists(testBibName))
		testTexName = os.path.join(pbConfig.dataPath, "tests_%s.tex"%today_ymd)
		texString = "\cite{empty}\citep{empty2}\citet{Gariazzo:2015rra}, \citet{Gariazzo:2017rra}\n"
		open(testTexName, "w").write(texString)
		self.assertEqual(open(testTexName).read(), texString)
		sampleList = [{"bibkey": "empty", "bibtex": '@Article{empty,\nauthor="me",\ntitle="no"\n}'}, {"bibkey": "empty2", "bibtex": '@Article{empty2,\nauthor="me2",\ntitle="yes"\n}'}]

		with patch('physbiblio.database.catsEntries.insert', return_value = True) as _ceins:
			with patch('physbiblio.database.entries.getByBibkey', side_effect = [
					[{"bibkey": "empty", "bibtex": '@Article{empty,\nauthor="me",\ntitle="no"\n}'}],
					[{"bibkey": "empty2", "bibtex": '@Article{empty2,\nauthor="me2",\ntitle="yes"\n}'}],
					[{"bibkey": "Gariazzo:2015rra", "bibtex": '@article{Gariazzo:2015rra,\nauthor= "Gariazzo, S. and others",\ntitle="{Light sterile neutrinos}",\n}'}],
					[{"bibkey": "Gariazzo:2015rra", "bibtex": '@article{Gariazzo:2015rra,\nauthor= "Gariazzo, S. and others",\ntitle="{Light sterile neutrinos}",\n}'}],
					[], []]) as _getbbibk:
				with patch('physbiblio.database.entries.getByBibtex', side_effect = [
						[{"bibkey": "empty", "bibtex": '@Article{empty,\nauthor="me",\ntitle="no"\n}'}],
						[{"bibkey": "empty2", "bibtex": '@Article{empty2,\nauthor="me2",\ntitle="yes"\n}'}],
						[], []]) as _getbbibt:
					with patch('physbiblio.database.entries.loadAndInsert',
							side_effect = ['@article{Gariazzo:2015rra,\nauthor= "Gariazzo, S. and others",\ntitle="{Light sterile neutrinos}",\n}', '']) as _mock:
						pBExport.exportForTexFile(testTexName, testBibName, overwrite = True, autosave = False)
		self.assertTrue(os.path.exists(testBibName))
		self.assertEqual(open(testBibName).read(), '%file written by PhysBiblio\n@Article{empty,\nauthor="me",\ntitle="no"\n}\n@Article{empty2,\nauthor="me2",\ntitle="yes"\n}\n@article{Gariazzo:2015rra,\nauthor= "Gariazzo, S. and others",\ntitle="{Light sterile neutrinos}",\n}\n')
		os.remove(testBibName)
		os.remove(testTexName)

if __name__=='__main__':
	unittest.main()



#!/usr/bin/env python
import sys
from PySide.QtCore import *
from PySide.QtGui  import *
import subprocess
import traceback

if sys.version_info[0] < 3:
	from StringIO import StringIO
else:
	from io import StringIO

try:
	#from physbiblio.database import *
	#from physbiblio.export import pBExport
	#import physbiblio.webimport.webInterf as webInt
	#from physbiblio.cli import cli as physBiblioCLI
	from physbiblio.config import pbConfig
	from physbiblio.gui.CommonClasses import *
	from physbiblio.errors import pBLogger, pBErrorManager
	from physbiblio.database import pBDB
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
try:
	if sys.version_info[0] < 3:
		import physbiblio.gui.Resources_pyside
	else:
		import physbiblio.gui.Resources_pyside3
except ImportError:
	print("Missing Resources_pyside: Run script update_resources.sh")

def askYesNo(message, title = "Question"):
	reply = QMessageBox.question(None, title, message, QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
	if reply == QMessageBox.Yes:
		return True
	if reply == QMessageBox.No:
		return False

def askFileName(parent = None, title = "Filename to use:", filter = ""):
	reply = QFileDialog.getOpenFileName(parent, title, "", filter)
	return reply[0]

def askFileNames(parent = None, title = "Filename to use:", filter = ""):
	reply = QFileDialog.getOpenFileNames(parent, title, "", filter)
	return reply[0]

def askSaveFileName(parent = None, title = "Filename to use:", filter = "", dir = ""):
	reply = QFileDialog.getSaveFileName(parent, title, dir, filter, options = QFileDialog.DontConfirmOverwrite)
	return reply[0]

def askDirName(parent = None, title = "Directory to use:", dir = ""):
	reply = QFileDialog.getExistingDirectory(parent, title, dir, options = QFileDialog.ShowDirsOnly)
	return reply

def askGenericText(message, title, parent = None):
	reply = QInputDialog.getText(parent, title, message)
	return reply[0]

def infoMessage(message, title = "Information"):
	reply = QMessageBox.information(None, title, message)

class configEditColumns(QDialog):
	def __init__(self, parent = None):
		super(configEditColumns, self).__init__(parent)
		self.excludeCols = ["crossref", "bibtex", "exp_paper", "lecture", "phd_thesis", "review", "proceeding", "book", "noUpdate"]
		self.moreCols = ["title", "author", "journal", "volume", "pages", "primaryclass", "booktitle", "reportnumber"]
		self.initUI()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.result = True
		self.selected = []
		for row in range(self.listSel.rowCount()):
			self.selected.append(self.listSel.item(row, 0).text())
		self.close()

	def initUI(self):
		self.layout = QGridLayout()
		self.setLayout(self.layout)

		self.listAll = MyDDTableWidget("Available columns")
		self.listSel = MyDDTableWidget("Selected columns")
		self.layout.addWidget(QLabel("Drag and drop items to order visible columns"), 0, 0, 1, 2)
		self.layout.addWidget(self.listAll, 1, 0)
		self.layout.addWidget(self.listSel, 1, 1)

		self.allItems = pBDB.descriptions["entries"].keys() + self.moreCols
		self.selItems = pbConfig.params["bibtexListColumns"]
		i=0
		for col in self.allItems:
			if col not in self.selItems and col not in self.excludeCols:
				item = QTableWidgetItem(col)
				self.listAll.insertRow(i)
				self.listAll.setItem(i, 0, item)
				i += 1
		for i, col in enumerate(self.selItems):
			item = QTableWidgetItem(col)
			self.listSel.insertRow(i)
			self.listSel.setItem(i, 0, item)

		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.layout.addWidget(self.acceptButton, 2, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.layout.addWidget(self.cancelButton, 2, 1)

class printText(QDialog):
	"""create a window for printing text of command line output"""
	stopped = Signal()

	def __init__(self, parent = None, title = "", progressBar = True, totStr = None, progrStr = None, noStopButton = False):
		super(printText, self).__init__(parent)
		self.message = None
		if title != "":
			self.title = title
		else:
			self.title = "Redirect print"
		self.setProgressBar = progressBar
		self.noStopButton = noStopButton
		self._want_to_close = False
		self.totString = totStr if totStr is not None else "emptyString"
		self.progressString = progrStr if progrStr is not None else "emptyString"
		self.initUI()

	def closeEvent(self, evnt):
		if self._want_to_close:
			super(printText, self).closeEvent(evnt)
		else:
			evnt.ignore()
			#self.setWindowState(QtCore.Qt.WindowMinimized)

	def initUI(self):
		self.setWindowTitle(self.title)

		grid = QGridLayout()
		grid.setSpacing(1)

		i = 0
		if self.message is not None:
			grid.addWidget(QLabel("%s"%self.message), 0, 0)
			i += 1

		#main text
		self.textEdit = QTextEdit()
		grid.addWidget(self.textEdit)

		if self.setProgressBar:
			self.progressBar = QProgressBar(self)
			grid.addWidget(self.progressBar)

		# cancel button...should learn how to connect it with a thread kill
		if self.noStopButton is not True:
			self.cancelButton = QPushButton('Stop', self)
			self.cancelButton.clicked.connect(self.stopExec)
			self.cancelButton.setAutoDefault(True)
			grid.addWidget(self.cancelButton)
		self.closeButton = QPushButton('Close', self)
		self.closeButton.clicked.connect(self.reject)
		self.closeButton.setDisabled(True)
		grid.addWidget(self.closeButton)

		self.setGeometry(100,100,600, 600)
		self.setLayout(grid)

		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

	def append_text(self,text):
		if self.setProgressBar:
			if self.totString in text:
				tot = [int(s) for s in text.split() if s.isdigit()][0]
				self.progressBarMax(tot)
			elif self.progressString in text:
				curr = [int(s) for s in text.split() if s.isdigit()][0]
				self.progressBar.setValue(curr)
		self.textEdit.moveCursor(QTextCursor.End)
		self.textEdit.insertPlainText( text )

	def progressBarMin(self, minimum):
		if self.setProgressBar:
			self.progressBar.setMinimum(minimum)

	def progressBarMax(self, maximum):
		if self.setProgressBar:
			self.progressBar.setMaximum(maximum)

	def stopExec(self):
		self.cancelButton.setDisabled(True)
		self.stopped.emit()

	def enableClose(self):
		self._want_to_close = True
		self.closeButton.setEnabled(True)

class searchReplaceDialog(QDialog):
	"""create a window for search and replace"""
	def __init__(self, parent = None):
		super(searchReplaceDialog, self).__init__(parent)
		self.initUI()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.result	= True
		self.close()

	def initUI(self):
		self.setWindowTitle('Search and replace')

		grid = QGridLayout()
		grid.setSpacing(1)

		#search
		grid.addWidget(QLabel("Search: "), 0, 0)
		self.searchEdit = QLineEdit("")
		grid.addWidget(self.searchEdit, 0, 1)

		#replace
		grid.addWidget(QLabel("Replace with: "), 1, 0)
		self.replaceEdit = QLineEdit("")
		grid.addWidget(self.replaceEdit, 1, 1)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		grid.addWidget(self.acceptButton, 2, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		grid.addWidget(self.cancelButton, 2, 1)

		self.setGeometry(100,100,400, 100)
		self.setLayout(grid)

		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

class advImportDialog(QDialog):
	"""create a window for the advanced import"""
	def __init__(self, parent = None):
		super(advImportDialog, self).__init__(parent)
		self.initUI()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.result	= True
		self.close()

	def initUI(self):
		self.setWindowTitle('Advanced import')

		grid = QGridLayout()
		grid.setSpacing(1)

		##search
		grid.addWidget(QLabel("Select method: "), 0, 0)
		self.comboMethod = MyComboBox(self,
			["INSPIRE-HEP", "arXiv", "DOI", "ISBN"],
			current = "INSPIRE-HEP")
		grid.addWidget(self.comboMethod, 0, 1)

		grid.addWidget(QLabel("Search string: "), 1, 0)
		self.searchStr = QLineEdit("")
		grid.addWidget(self.searchStr, 1, 1)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		grid.addWidget(self.acceptButton, 2, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		grid.addWidget(self.cancelButton, 2, 1)

		self.setGeometry(100,100,400, 100)
		self.setLayout(grid)
		self.searchStr.setFocus()

		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

class advImportSelect(QDialog):
	"""create a window for the advanced import"""
	def __init__(self, bibs=[], parent = None):
		super(advImportSelect, self).__init__(parent)
		self.bibs = bibs
		self.oneNew = False
		self.checkBoxes = []
		self.initUI()

	def onCancel(self):
		self.result	= False
		self.close()

	def onOk(self):
		self.result	= True
		self.close()

	def initUI(self):
		self.setWindowTitle('Advanced import - results')

		grid = QGridLayout()
		grid.setSpacing(1)

		grid.addWidget(QLabel("This is the list of elements found.\nSelect the ones that you want to import:\n"), 0, 0, 1, 2)
		##search
		i = 0
		for bk, elDic in self.bibs.items():
			el = elDic["bibpars"]
			if elDic["exist"] is False:
				self.checkBoxes.append(QCheckBox(bk, self))
				self.checkBoxes[i].toggle()
				grid.addWidget(self.checkBoxes[i], i+1, 0)
				try:
					title = el["title"]
				except:
					title = "Title not found"
				try:
					authors = el["author"]
				except:
					authors = "Authors not found"
				try:
					arxiv = el["arxiv"]
				except:
					try:
						arxiv = el["eprint"]
					except:
						arxiv = "arXiv number not found"
				grid.addWidget(QLabel("%s\n%s\n%s"%(title, authors, arxiv)), i+1, 1)
				self.oneNew = True
			else:
				self.checkBoxes.append(QCheckBox(bk, self))
				self.checkBoxes[i].setDisabled(True)
				grid.addWidget(self.checkBoxes[i], i+1, 0)
				grid.addWidget(QLabel("Already existing"), i+1, 1)
			i += 1

		i += 2
		grid.addWidget(QLabel("\n"), i-1, 1)
		if self.oneNew:
			grid.addWidget(QLabel("Ask categories at the end?"), i, 1)
			self.askCats = QCheckBox("", self)
			self.askCats.toggle()
			grid.addWidget(self.askCats, i, 0)

			# OK button
			self.acceptButton = QPushButton('OK', self)
			self.acceptButton.clicked.connect(self.onOk)
			grid.addWidget(self.acceptButton, i+1, 0)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		grid.addWidget(self.cancelButton, i+1, 1)

		self.setGeometry(100,100,400, 100)
		self.setLayout(grid)

		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())



#!/usr/bin/env python
"""
Test file for the physbiblio.webimport subpackage.

This file is part of the physbiblio package.
"""
import sys, datetime, traceback, os

if sys.version_info[0] < 3:
	import unittest2 as unittest
else:
	import unittest

try:
	from physbiblio.setuptests import *
	from physbiblio.webimport.webInterf import physBiblioWeb
	from physbiblio.webimport.inspireoai import get_journal_ref_xml
except ImportError:
    print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
    raise
except Exception:
	print(traceback.format_exc())

@unittest.skipIf(skipOnlineTests, "Online tests")
class TestWebImportMethods(unittest.TestCase):
	"""
	Test the functions that import entries from the web.
	Should not fail if everything works fine.
	"""

	def test_methods_success(self):
		"""Test webimport with known results"""
		print(physBiblioWeb.webSearch.keys())
		tests = {
			"arxiv": ["1507.08204", """@Article{1507.08204,
         title = "{Light sterile neutrinos}",
          year = "2015",
 archiveprefix = "arXiv",
  primaryclass = "hep-ph",
           doi = "10.1088/0954-3899/43/3/033001",
      abstract = "{The theory and phenomenology of light sterile neutrinos at the eV mass scale is reviewed. The reactor, Gallium and LSND anomalies are briefly described and interpreted as indications of the existence of short-baseline oscillations which require the existence of light sterile neutrinos. The global fits of short-baseline oscillation data in 3+1 and 3+2 schemes are discussed, together with the implications for beta-decay and neutrinoless double-beta decay. The cosmological effects of light sterile neutrinos are briefly reviewed and the implications of existing cosmological data are discussed. The review concludes with a summary of future perspectives.}",
         arxiv = "1507.08204",
       authors = "S. Gariazzo and C. Giunti and M. Laveder and Y. F. Li and E. M. Zavanin",
}"""],
			"doi": ["10.1088/0954-3899/43/3/033001", """@article{Gariazzo_2015,
	doi = {10.1088/0954-3899/43/3/033001},
	url = {https://doi.org/10.1088%2F0954-3899%2F43%2F3%2F033001},
	year = 2015,
	month = {mar},
	publisher = {{IOP} Publishing},
	volume = {43},
	number = {3},
	pages = {033001},
	author = {S Gariazzo and C Giunti and M Laveder and Y F Li and E M Zavanin},
	title = {Light sterile neutrinos},
	journal = {Journal of Physics G: Nuclear and Particle Physics}
}"""],
			"inspire": ["Gariazzo:2015rra", """@article{Gariazzo:2015rra,
      author         = "Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y. F.
                        and Zavanin, E. M.",
      title          = "{Light sterile neutrinos}",
      journal        = "J. Phys.",
      volume         = "G43",
      year           = "2016",
      pages          = "033001",
      doi            = "10.1088/0954-3899/43/3/033001",
      eprint         = "1507.08204",
      archivePrefix  = "arXiv",
      primaryClass   = "hep-ph",
      SLACcitation   = "%%CITATION = ARXIV:1507.08204;%%"
}"""],
			"inspireoai": ["1385583",
				{'doi': '10.1088/0954-3899/43/3/033001', 'arxiv': '1507.08204', 'bibkey': 'Gariazzo:2015rra', 'ads': '2015JPhG...43c3001G', 'journal': 'J.Phys.', 'volume': 'G43', 'year': '2016', 'pages': '033001', 'firstdate': '2015-07-29', 'pubdate': '2016-01-13', 'author': 'Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y.F. and Zavanin, E.M.', 'collaboration': None, 'primaryclass': 'hep-ph', 'archiveprefix': 'arXiv', 'eprint': '1507.08204', 'title': 'Light sterile neutrinos', 'isbn': None, 'ENTRYTYPE': 'article', 'oldkeys': '', 'bibtex': '@Article{Gariazzo:2015rra,\n        author = "Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y.F. and Zavanin, E.M.",\n         title = "{Light sterile neutrinos}",\n       journal = "J.Phys.",\n        volume = "G43",\n          year = "2016",\n         pages = "033001",\n archiveprefix = "arXiv",\n  primaryclass = "hep-ph",\n        eprint = "1507.08204",\n           doi = "10.1088/0954-3899/43/3/033001",\n}\n\n', 'id': '1385583', 'link': 'http://dx.doi.org/10.1088/0954-3899/43/3/033001', 'reportnumber': None}],
			"isbn": ["9780198508717", """@book{9780198508717,
  Author = {Carlo Giunti, Chung W. Kim},
  Title = {Fundamentals of Neutrino Physics and Astrophysics},
  Publisher = {OXFORD UNIV PR},
  Year = {2007},
  Date = {2007-05-11},
  PageTotal = {710 Seiten},
  EAN = {9780198508717},
  ISBN = {0198508719},
  URL = {https://www.ebook.de/de/product/7616244/carlo_giunti_chung_w_kim_fundamentals_of_neutrino_physics_and_astrophysics.html}
}"""],
			}
		for method, strings in tests.items():
			print(method)
			if method == "inspireoai":
				self.assertEqual(physBiblioWeb.webSearch[method].retrieveOAIData(strings[0]), strings[1])
			else:
				self.assertEqual(physBiblioWeb.webSearch[method].retrieveUrlFirst(strings[0]).strip(), strings[1].strip())
				self.assertEqual(physBiblioWeb.webSearch[method].retrieveUrlAll(strings[0]).strip(), strings[1].strip())
		self.assertEqual(physBiblioWeb.webSearch["inspire"].retrieveInspireID(tests["inspire"][0]), u'1385583')

	def test_methods_insuccess(self):
		"""Test webimport using missing and/or invalid identifiers"""
		print(physBiblioWeb.webSearch.keys())
		tests = {
			"arxiv": ["1801.15000", ""],
			"doi": ["10.1088/9999-3899/43/a/033001", ""],
			"inspire": ["Gariazzo:2014rra", ""],
			"isbn": ["978019850871a", ""],
			}
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].retrieveOAIData("1110620"),
			{'doi': None, 'arxiv': None, 'bibkey': None, 'ads': None, 'journal': None, 'volume': None, 'year': None, 'pages': None, 'firstdate': None, 'pubdate': None, 'author': ' and Tauber, Jan', 'collaboration': 'Planck', 'title': 'Anisotropies of the Cosmic Background Radiation Field', 'isbn': None, 'ENTRYTYPE': 'article', 'oldkeys': '', 'bibtex': '@Article{,\n        author = " and Tauber, Jan",\n collaboration = "Planck",\n         title = "{Anisotropies of the Cosmic Background Radiation Field}",\n}\n\n', 'id': '1110620', 'archiveprefix': None, 'eprint': None, 'link': None, 'primaryclass': None, 'reportnumber': None,})
		self.assertFalse(physBiblioWeb.webSearch["inspireoai"].retrieveOAIData("9999999"))
		for method, strings in tests.items():
			print(method)
			res=physBiblioWeb.webSearch[method].retrieveUrlFirst(strings[0])
			print(res)
			self.assertEqual(res.strip(), strings[1].strip())
			self.assertEqual(physBiblioWeb.webSearch[method].retrieveUrlAll(strings[0]).strip(), strings[1].strip())
		self.assertEqual(physBiblioWeb.webSearch["inspire"].retrieveInspireID(tests["inspire"][0]), "")

	def test_inspireoai_other(self):
		"""test auxiliary functions in inspireoai module"""
		marcxmlRecord1 = physBiblioWeb.webSearch["inspireoai"].oai.getRecord(metadataPrefix = 'marcxml', identifier = "oai:inspirehep.net:1385583")[1]
		marcxmlRecord2 = physBiblioWeb.webSearch["inspireoai"].oai.getRecord(metadataPrefix = 'marcxml', identifier = "oai:inspirehep.net:1414175")[1]
		self.assertEqual(get_journal_ref_xml(marcxmlRecord1),
			(['J.Phys.'], ['G43'], ['2016'], ['033001'], [None], [None], [None], [None]))
		self.assertEqual(get_journal_ref_xml(marcxmlRecord2),
			([None], [None], ['2017'], ['469-475'], [None], [None], [None], ['C15-08-20']))
		dict1a = physBiblioWeb.webSearch["inspireoai"].readRecord(marcxmlRecord1)
		dict1b = physBiblioWeb.webSearch["inspireoai"].readRecord(marcxmlRecord1, readConferenceTitle = True)
		self.assertEqual(dict1a, dict1b)
		self.assertEqual(dict1a,
			{'doi': '10.1088/0954-3899/43/3/033001', 'arxiv': '1507.08204', 'bibkey': 'Gariazzo:2015rra', 'ads': '2015JPhG...43c3001G', 'journal': 'J.Phys.', 'volume': 'G43', 'year': '2016', 'pages': '033001', 'firstdate': '2015-07-29', 'pubdate': '2016-01-13', 'author': 'Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y.F. and Zavanin, E.M.', 'collaboration': None, 'primaryclass': 'hep-ph', 'archiveprefix': 'arXiv', 'eprint': '1507.08204', 'reportnumber': None, 'title': 'Light sterile neutrinos', 'isbn': None, 'ENTRYTYPE': 'article', 'oldkeys': '', 'link': 'http://dx.doi.org/10.1088/0954-3899/43/3/033001', 'bibtex': '@Article{Gariazzo:2015rra,\n        author = "Gariazzo, S. and Giunti, C. and Laveder, M. and Li, Y.F. and Zavanin, E.M.",\n         title = "{Light sterile neutrinos}",\n       journal = "J.Phys.",\n        volume = "G43",\n          year = "2016",\n         pages = "033001",\n archiveprefix = "arXiv",\n  primaryclass = "hep-ph",\n        eprint = "1507.08204",\n           doi = "10.1088/0954-3899/43/3/033001",\n}\n\n'})
		dict2 = physBiblioWeb.webSearch["inspireoai"].readRecord(marcxmlRecord2, readConferenceTitle = True)
		self.assertEqual(dict2, {'doi': '10.1142/9789813224568_0076', 'arxiv': '1601.01475', 'bibkey': 'Gariazzo:2016ehl', 'ads': None, 'journal': None, 'volume': None, 'year': '2017', 'pages': '469-475', 'firstdate': '2016-01-07', 'pubdate': '2017', 'author': 'Gariazzo, Stefano', 'collaboration': None, 'primaryclass': 'astro-ph.CO', 'archiveprefix': 'arXiv', 'eprint': '1601.01475', 'reportnumber': None, 'title': 'Light Sterile Neutrinos In Cosmology', 'isbn': None, 'booktitle': 'Proceedings, 17th Lomonosov Conference on Elementary Particle Physics: Moscow, Russia, August 20-26, 2015', 'ENTRYTYPE': 'inproceedings', 'oldkeys': '', 'link': 'http://dx.doi.org/10.1142/9789813224568_0076', 'bibtex': '@Inproceedings{Gariazzo:2016ehl,\n        author = "Gariazzo, Stefano",\n         title = "{Light Sterile Neutrinos In Cosmology}",\n     booktitle = "{Proceedings, 17th Lomonosov Conference on Elementary Particle Physics: Moscow, Russia, August 20-26, 2015}",\n          year = "2017",\n         pages = "469-475",\n archiveprefix = "arXiv",\n  primaryclass = "astro-ph.CO",\n        eprint = "1601.01475",\n           doi = "10.1142/9789813224568_0076",\n}\n\n'})
		dict1a["id"] = "1385583"
		dict2["id"] = "1414175"
		bibtex1 = '@article{abc,\nauthor="me",}'
		bibtex2 = '@article{abc,'
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].updateBibtex(dict2, bibtex1), bibtex1)
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].updateBibtex(dict1a, bibtex2), bibtex2)
		self.assertEqual(physBiblioWeb.webSearch["inspireoai"].updateBibtex(dict1a, bibtex1),
			'@Article{abc,\n        author = "me",\n       journal = "J.Phys.",\n        volume = "G43",\n          year = "2016",\n         pages = "033001",\n           doi = "10.1088/0954-3899/43/3/033001",\n}\n\n')

	@unittest.skipIf(skipOAITests, "Online tests with OAI")
	def test_inspireoai(self):
		"""test retrieve daily data from inspireOAI"""
		date1 = (datetime.date.today() - datetime.timedelta(1)).strftime("%Y-%m-%d")
		date2 = datetime.date.today().strftime("%Y-%m-%d")
		yren, monen, dayen = date1.split('-')
		yrst, monst, dayst = date2.split('-')
		date1 = datetime.datetime(int(yren), int(monen), int(dayen))
		date2 = datetime.datetime(int(yrst), int(monst), int(dayst))
		result = physBiblioWeb.webSearch["inspireoai"].retrieveOAIUpdates(date1, date2)
		self.assertEqual(type(result), list)
		print(len(result), result[0])

if __name__=='__main__':
	unittest.main()



#!/usr/bin/env python

import sys, time
import os
from PySide.QtCore import *
from PySide.QtGui  import *

try:
	from physbiblio.config import pbConfig
	from physbiblio.database import *
	from physbiblio.gui.ErrorManager import pBGUILogger
	from physbiblio.gui.DialogWindows import *
	from physbiblio.gui.CommonClasses import *
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")

def editProf(parent, statusBarObject):
	newProfWin = editProfile(parent)
	newProfWin.exec_()
	data = {}
	if newProfWin.result:
		newProfiles = {}
		for currEl in newProfWin.elements:
			name = currEl["n"].text()
			if currEl["r"].isChecked() and name != "":
				pbConfig.defProf = name
			if name in pbConfig.profiles.keys():
				pbConfig.profiles[name]["d"] = currEl["d"].text()
				if currEl["x"].isChecked() and \
						askYesNo("Do you really want to cancel the profile '%s'?\nThe action cannot be undone!\nThe corresponding database will not be erased."%name):
					del pbConfig.profiles[name]
					try:
						os.remove(os.path.join(pbConfig.configPath, currEl["f"].text()))
					except OSError:
						pBGUILogger.warning("Impossible to cancel the profile file %s."%currEl["f"])
			else:
				if name.strip() != "":
					pbConfig.profiles[name] = {}
					pbConfig.profiles[name]["d"] = currEl["d"].text()
					fname = currEl["f"].text().split(os.sep)[-1] + ".cfg"
					fname = fname.replace(".cfg.cfg", ".cfg")
					pbConfig.profiles[name]["f"] = fname
					open(os.path.join(pbConfig.configPath, fname), "a").close()
					pBGUILogger.info("New profile created.\nYou should configure it properly before use!\n(switch to it and open the configuration)")
		pbConfig.writeProfiles()
	else:
		message = "No modifications"
	try:
		statusBarObject.StatusBarMessage(message)
	except:
		pass

class selectProfiles(QDialog):
	def __init__(self, parent = None, message = None):
		super(selectProfiles, self).__init__(parent)
		self.parent = parent
		self.message = message
		self.initUI()
	
	def onCancel(self):
		self.result	= False
		self.close()

	def onLoad(self):
		prof, desc = self.combo.currentText().split(" -- ")
		newProfile = pbConfig.profiles[prof]
		if newProfile != pbConfig.defaultProfile:
			print("[config] Changing profile...")
			pbConfig.reInit(prof, newProfile)
			if pbConfig.needFirstConfiguration:
				infoMessage("Missing configuration file.\nYou should verify the configuration now.")
				self.parent.config()
			pBDB.reOpenDB(pbConfig.params['mainDatabaseName'])

		self.parent.reloadConfig()
		self.parent.reloadMainContent()
		self.close()

	def initUI(self):
		self.setWindowTitle('Select profile')

		grid = QGridLayout()
		grid.setSpacing(1)

		i = 0
		if self.message is not None:
			grid.addWidget(QLabel("%s"%self.message), 0, 0)
			i += 1

		grid.addWidget(QLabel("Available profiles: "), i, 0)
		self.combo = MyComboBox(self,
			["%s -- %s"%(p, pbConfig.profiles[p]["d"]) for p in pbConfig.profiles.keys()],
			current = "%s -- %s"%(pbConfig.defProf, pbConfig.defaultProfile["d"]))
		grid.addWidget(self.combo, i, 1)
		i += 1

		# cancel button
		self.loadButton = QPushButton('Load', self)
		self.loadButton.clicked.connect(self.onLoad)
		grid.addWidget(self.loadButton, i+1, 0)
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		grid.addWidget(self.cancelButton, i+1, 1)

		#self.setGeometry(100,100,300, 30*i)
		self.setLayout(grid)

		qr = self.frameGeometry()
		cp = QDesktopWidget().availableGeometry().center()
		qr.moveCenter(cp)
		self.move(qr.topLeft())

class editProfile(editObjectWindow):
	"""create a window for editing or creating a profile"""
	def __init__(self, parent = None):
		super(editProfile, self).__init__(parent)
		self.createForm()

	def createForm(self):
		self.setWindowTitle('Edit profile')

		i = 0
		labels = [ QLabel("Default"), QLabel("Short name"), QLabel("Filename"), QLabel("Description"), QLabel("Delete?") ]
		for i,e in enumerate(labels):
			self.currGrid.addWidget(e, 0, i)

		self.def_group = QButtonGroup(self.currGrid)
		self.elements = []
		for k, prof in pbConfig.profiles.items():
			i += 1
			tempEl = {}
			tempEl["r"] = QRadioButton("")
			self.def_group.addButton(tempEl["r"])
			if pbConfig.defProf == k:
				tempEl["r"].setChecked(True)
			else:
				tempEl["r"].setChecked(False)
			self.currGrid.addWidget(tempEl["r"], i, 0)

			tempEl["n"] = QLineEdit(k)
			tempEl["f"] = QLineEdit(prof["f"].split(os.sep)[-1])
			tempEl["n"].setReadOnly(True)
			tempEl["f"].setReadOnly(True)
			tempEl["d"] = QLineEdit(prof["d"])
			self.currGrid.addWidget(tempEl["n"], i, 1)
			self.currGrid.addWidget(tempEl["f"], i, 2)
			self.currGrid.addWidget(tempEl["d"], i, 3)
			tempEl["x"] = QCheckBox("", self)
			self.currGrid.addWidget(tempEl["x"], i, 4)
			self.elements.append(tempEl)
		i += 2
		tempEl = {}
		self.currGrid.addWidget(QLabel("Add new?"), i-1, 0)
		tempEl["r"] = QRadioButton("")
		self.def_group.addButton(tempEl["r"])
		tempEl["r"].setChecked(False)
		self.currGrid.addWidget(tempEl["r"], i, 0)

		tempEl["n"] = QLineEdit("")
		tempEl["f"] = QLineEdit("")
		tempEl["d"] = QLineEdit("")
		self.currGrid.addWidget(tempEl["n"], i, 1)
		self.currGrid.addWidget(tempEl["f"], i, 2)
		self.currGrid.addWidget(tempEl["d"], i, 3)
		self.elements.append(tempEl)

		# OK button
		self.acceptButton = QPushButton('OK', self)
		self.acceptButton.clicked.connect(self.onOk)
		self.currGrid.addWidget(self.acceptButton, i+1, 1)

		# cancel button
		self.cancelButton = QPushButton('Cancel', self)
		self.cancelButton.clicked.connect(self.onCancel)
		self.cancelButton.setAutoDefault(True)
		self.currGrid.addWidget(self.cancelButton, i+1, 0)

		self.setGeometry(100,100,700, 25*i)
		self.centerWindow()



"""
Functions that should run only when PhysBiblio is opened for the first time.

This file is part of the physbiblio package.
"""
import os, traceback
try:
	from physbiblio.errors import pBLogger
	from physbiblio.config import pbConfig
	import physbiblio.tablesDef
except ImportError:
	print("Could not find physbiblio and its contents: configure your PYTHONPATH!")
	print(traceback.format_exc())
	raise

def createTables(database):
	"""
	Create tables for the database (and insert the default categories), if it is missing.
	"""
	for q in database.tableFields.keys():
		command="CREATE TABLE "+q+" (\n"
		first=True
		for el in database.tableFields[q]:
			if first:
				first=False
			else:
				command+=",\n"
			command+=" ".join(el)
		command+=");"
		pBLogger.info(command+"\n")
		if not database.connExec(command):
			pBLogger.critical("Create %s failed"%q, exec_info = True)
	command="""
	INSERT into categories (idCat, name, description, parentCat, ord)
		values (0,"Main","This is the main category. All the other ones are subcategories of this one",0,0),
		(1,"Tags","Use this category to store tags (such as: ongoing projects, temporary cats,...)",0,0)
		"""
	pBLogger.info(command+"\n")
	if not database.connExec(command):
		pBLogger.exception("Insert main categories failed", exec_info = True)
	database.commit()
		
